@@ -75,6 +75,7 @@ from _drgn import (
     Type,
     TypeEnumerator,
     TypeKind,
+    TypeKindSet,
     TypeMember,
     TypeParameter,
     TypeTemplateParameter,
@@ -129,6 +130,7 @@ __all__ = (
     "Type",
     "TypeEnumerator",
     "TypeKind",
+    "TypeKindSet",
     "TypeMember",
     "TypeParameter",
     "TypeTemplateParameter",
