@@ -924,8 +924,8 @@ static void read_phdr(const void *phdr_buf, size_t i, bool is_64_bit,
 	}
 }
 
-static const char *read_build_id(const char *buf, size_t buf_len,
-				 uint64_t align, bool bswap,
+static const char *read_build_id(const void *buf, size_t buf_len,
+				 unsigned int align, bool bswap,
 				 size_t *len_ret)
 {
 	/*
@@ -934,39 +934,18 @@ static const char *read_build_id(const char *buf, size_t buf_len,
 	 */
 	static const uint32_t build_id_min_size = 2;
 	static const uint32_t build_id_max_size = 1024;
-	/* Elf32_Nhdr is the same as Elf64_Nhdr. */
-	Elf64_Nhdr nhdr;
-	const char *p = buf;
-	while (buf + buf_len - p >= sizeof(nhdr)) {
-		memcpy(&nhdr, p, sizeof(nhdr));
-		if (bswap) {
-			nhdr.n_namesz = bswap_32(nhdr.n_namesz);
-			nhdr.n_descsz = bswap_32(nhdr.n_descsz);
-			nhdr.n_type = bswap_32(nhdr.n_type);
-		}
-		p += sizeof(nhdr);
-
-		uint64_t namesz = (nhdr.n_namesz + align - 1) & ~(align - 1);
-		if (namesz > buf + buf_len - p)
-			return NULL;
-		const char *name = p;
-		p += namesz;
-
+	Elf32_Nhdr nhdr;
+	const char *name;
+	const void *desc;
+	while (next_elf_note(&buf, &buf_len, align, bswap, &nhdr, &name, &desc)) {
 		if (nhdr.n_namesz == sizeof("GNU") &&
 		    memcmp(name, "GNU", sizeof("GNU")) == 0 &&
 		    nhdr.n_type == NT_GNU_BUILD_ID &&
 		    nhdr.n_descsz >= build_id_min_size &&
 		    nhdr.n_descsz <= build_id_max_size) {
-			if (nhdr.n_descsz > buf + buf_len - p)
-				return NULL;
 			*len_ret = nhdr.n_descsz;
-			return p;
+			return desc;
 		}
-
-		uint64_t descsz = (nhdr.n_descsz + align - 1) & ~(align - 1);
-		if (descsz > buf + buf_len - p)
-			return NULL;
-		p += descsz;
 	}
 	return NULL;
 }
@@ -1104,7 +1083,8 @@ userspace_core_identify_file(struct drgn_program *prog,
 			}
 			ret->build_id = read_build_id(core->segment_buf,
 						      phdr.p_filesz,
-						      phdr.p_align, arg.bswap,
+						      phdr.p_align == 8 ? 8 : 4,
+						      arg.bswap,
 						      &ret->build_id_len);
 			if (ret->build_id)
 				break;
@@ -1664,7 +1644,7 @@ static struct drgn_error *relocate_elf_file(Elf *elf)
 		if (r < 0)
 			return drgn_error_libelf();
 		if (r) {
-			Elf_Scn *scn = elf_getscn(elf, reloc_shdr->sh_info);
+			scn = elf_getscn(elf, reloc_shdr->sh_info);
 			if (!scn)
 				return drgn_error_libelf();
 			GElf_Shdr *shdr, shdr_mem;
@@ -2024,6 +2004,148 @@ err:
 	goto out;
 }
 
+struct elf_symbols_search_arg {
+	const char *name;
+	uint64_t address;
+	enum drgn_find_symbol_flags flags;
+	struct drgn_error *err;
+	struct drgn_symbol_result_builder *builder;
+};
+
+static bool elf_symbol_match(struct elf_symbols_search_arg *arg, GElf_Addr addr,
+			 const GElf_Sym *sym, const char *name)
+{
+	if ((arg->flags & DRGN_FIND_SYMBOL_NAME) && strcmp(name, arg->name) != 0)
+		return false;
+	if ((arg->flags & DRGN_FIND_SYMBOL_ADDR) &&
+	    (arg->address < addr || arg->address >= addr + sym->st_size))
+		return false;
+	return true;
+}
+
+static bool elf_symbol_store_match(struct elf_symbols_search_arg *arg,
+				   GElf_Sym *elf_sym, GElf_Addr addr,
+				   const char *name)
+{
+	struct drgn_symbol *sym;
+	if (arg->flags == (DRGN_FIND_SYMBOL_ONE | DRGN_FIND_SYMBOL_NAME)) {
+		int binding = GELF_ST_BIND(elf_sym->st_info);
+		/*
+		 * The order of precedence is
+		 * GLOBAL = UNIQUE > WEAK > LOCAL = everything else
+		 *
+		 * If we found a global or unique symbol, return it
+		 * immediately. If we found a weak symbol, then save it,
+		 * which may overwrite a previously found weak or local
+		 * symbol. Otherwise, save the symbol only if we haven't
+		 * found another symbol.
+		 */
+		if (binding != STB_GLOBAL
+		    && binding != STB_GNU_UNIQUE
+		    && binding != STB_WEAK
+		    && drgn_symbol_result_builder_count(arg->builder) > 0)
+			return false;
+		sym = malloc(sizeof(*sym));
+		if (!sym) {
+			arg->err = &drgn_enomem;
+			return true;
+		}
+		drgn_symbol_from_elf(name, addr, elf_sym, sym);
+		if (!drgn_symbol_result_builder_add(arg->builder, sym)) {
+			arg->err = &drgn_enomem;
+			drgn_symbol_destroy(sym);
+		}
+
+		/* Abort on error, or short-circuit if we found a global or
+		 * unique symbol */
+		return (arg->err || sym->binding == DRGN_SYMBOL_BINDING_GLOBAL
+			|| sym->binding == DRGN_SYMBOL_BINDING_UNIQUE);
+	} else {
+		sym = malloc(sizeof(*sym));
+		if (!sym) {
+			arg->err = &drgn_enomem;
+			return true;
+		}
+		drgn_symbol_from_elf(name, addr, elf_sym, sym);
+		if (!drgn_symbol_result_builder_add(arg->builder, sym)) {
+			arg->err = &drgn_enomem;
+			drgn_symbol_destroy(sym);
+		}
+		/* Abort on error, or short-circuit for single lookup */
+		return (arg->err || (arg->flags & DRGN_FIND_SYMBOL_ONE));
+	}
+}
+
+static int elf_symbols_search_cb(Dwfl_Module *dwfl_module, void **userdatap,
+			     const char *module_name, Dwarf_Addr base,
+			     void *cb_arg)
+{
+	struct elf_symbols_search_arg *arg = cb_arg;
+
+	int symtab_len = dwfl_module_getsymtab(dwfl_module);
+	if (symtab_len == -1)
+		return DWARF_CB_OK;
+
+	/* Ignore the zeroth null symbol */
+	for (int i = 1; i < symtab_len; i++) {
+		GElf_Sym elf_sym;
+		GElf_Addr elf_addr;
+		const char *name = dwfl_module_getsym_info(dwfl_module, i,
+							   &elf_sym, &elf_addr,
+							   NULL, NULL, NULL);
+		if (!name || !elf_symbol_match(arg, elf_addr, &elf_sym, name))
+			continue;
+		if (elf_symbol_store_match(arg, &elf_sym, elf_addr, name))
+			return DWARF_CB_ABORT;
+	}
+	return DWARF_CB_OK;
+}
+
+static struct drgn_error *
+elf_symbols_search(const char *name, uint64_t addr, enum drgn_find_symbol_flags flags,
+		   void *data, struct drgn_symbol_result_builder *builder)
+{
+	Dwfl_Module *dwfl_module = NULL;
+	struct drgn_program *prog = data;
+	struct elf_symbols_search_arg arg = {
+		.name = name,
+		.address = addr,
+		.flags = flags,
+		.err = NULL,
+		.builder = builder,
+	};
+
+	if (arg.flags & DRGN_FIND_SYMBOL_ADDR) {
+		dwfl_module = dwfl_addrmodule(prog->dbinfo.dwfl, arg.address);
+		if (!dwfl_module)
+			return NULL;
+	}
+
+	if ((arg.flags & (DRGN_FIND_SYMBOL_ADDR | DRGN_FIND_SYMBOL_ONE))
+	    == (DRGN_FIND_SYMBOL_ADDR | DRGN_FIND_SYMBOL_ONE)) {
+		GElf_Off offset;
+		GElf_Sym elf_sym;
+		const char *sym_name = dwfl_module_addrinfo(dwfl_module, addr,
+							    &offset, &elf_sym,
+							    NULL, NULL, NULL);
+		if (!sym_name)
+			return NULL;
+		struct drgn_symbol *sym = malloc(sizeof(*sym));
+		if (!sym)
+			return &drgn_enomem;
+		drgn_symbol_from_elf(sym_name, addr - offset, &elf_sym, sym);
+		if (!drgn_symbol_result_builder_add(builder, sym)) {
+			arg.err = &drgn_enomem;
+			drgn_symbol_destroy(sym);
+		}
+	} else if (dwfl_module) {
+		elf_symbols_search_cb(dwfl_module, NULL, NULL, 0, &arg);
+	} else {
+		dwfl_getmodules(prog->dbinfo.dwfl, elf_symbols_search_cb, &arg, 0);
+	}
+	return arg.err;
+}
+
 bool drgn_debug_info_is_indexed(struct drgn_debug_info *dbinfo,
 				const char *name)
 {
@@ -2039,11 +2161,24 @@ void drgn_debug_info_init(struct drgn_debug_info *dbinfo,
 	// unlikely to fail anwyays, so don't bother propagating an error up.
 	if (!dbinfo->dwfl)
 		abort();
-	drgn_program_add_type_finder_impl(prog, &dbinfo->type_finder,
-					  drgn_debug_info_find_type, dbinfo);
-	drgn_program_add_object_finder_impl(prog, &dbinfo->object_finder,
-					    drgn_debug_info_find_object,
-					    dbinfo);
+	const struct drgn_type_finder_ops type_finder_ops = {
+		.find = drgn_debug_info_find_type,
+	};
+	drgn_program_register_type_finder_impl(prog, &dbinfo->type_finder,
+					       "dwarf", &type_finder_ops,
+					       dbinfo, 0);
+	const struct drgn_object_finder_ops object_finder_ops = {
+		.find = drgn_debug_info_find_object,
+	};
+	drgn_program_register_object_finder_impl(prog, &dbinfo->object_finder,
+						 "dwarf", &object_finder_ops,
+						 dbinfo, 0);
+	const struct drgn_symbol_finder_ops symbol_finder_ops = {
+		.find = elf_symbols_search,
+	};
+	drgn_program_register_symbol_finder_impl(prog, &dbinfo->symbol_finder,
+						 "elf", &symbol_finder_ops,
+						 prog, 0);
 	drgn_module_table_init(&dbinfo->modules);
 	c_string_set_init(&dbinfo->module_names);
 	drgn_dwarf_info_init(dbinfo);
