@@ -35,11 +35,16 @@ static struct drgn_error *drgn_platform_from_kdump(kdump_ctx_t *ctx,
 		arch = &arch_info_arm;
 	else if (strcmp(str, KDUMP_ARCH_PPC64) == 0)
 		arch = &arch_info_ppc64;
-	/* libkdumpfile doesn't support RISC-V */
 	else if (strcmp(str, KDUMP_ARCH_S390X) == 0)
 		arch = &arch_info_s390x;
 	else if (strcmp(str, KDUMP_ARCH_S390) == 0)
 		arch = &arch_info_s390;
+#if KDUMPFILE_VERSION >= KDUMPFILE_MKVER(0, 5, 4)
+	else if (strcmp(str, KDUMP_ARCH_RISCV64) == 0)
+		arch = &arch_info_riscv64;
+	else if (strcmp(str, KDUMP_ARCH_RISCV32) == 0)
+		arch = &arch_info_riscv32;
+#endif
 	else
 		arch = &arch_info_unknown;
 
@@ -63,6 +68,45 @@ static struct drgn_error *drgn_platform_from_kdump(kdump_ctx_t *ctx,
 	return NULL;
 }
 
+static struct drgn_error *drgn_platform_to_kdump(kdump_ctx_t *ctx,
+						 const struct drgn_platform *platform)
+{
+	kdump_status ks;
+	char *arch_str = NULL;
+
+	if (platform->arch == &arch_info_x86_64)
+		arch_str = KDUMP_ARCH_X86_64;
+	else if (platform->arch == &arch_info_i386)
+		arch_str = KDUMP_ARCH_IA32;
+	else if (platform->arch == &arch_info_aarch64)
+		arch_str = KDUMP_ARCH_AARCH64;
+	else if (platform->arch == &arch_info_arm)
+		arch_str = KDUMP_ARCH_ARM;
+	else if (platform->arch == &arch_info_ppc64)
+		arch_str = KDUMP_ARCH_PPC64;
+	else if (platform->arch == &arch_info_s390x)
+		arch_str = KDUMP_ARCH_S390X;
+	else if (platform->arch == &arch_info_s390)
+		arch_str = KDUMP_ARCH_S390;
+#if KDUMPFILE_VERSION >= KDUMPFILE_MKVER(0, 5, 4)
+	else if (platform->arch == &arch_info_riscv64)
+		arch_str = KDUMP_ARCH_RISCV64;
+	else if (platform->arch == &arch_info_riscv32)
+		arch_str = KDUMP_ARCH_RISCV32;
+#endif
+
+	if (arch_str) {
+		ks = kdump_set_string_attr(ctx, KDUMP_ATTR_ARCH_NAME, arch_str);
+		if (ks != KDUMP_OK) {
+			return drgn_error_format(DRGN_ERROR_OTHER,
+						 "kdump_set_string_attr(\"%s\"): %s",
+						 arch_str, kdump_get_err(ctx));
+		}
+	}
+
+	return NULL;
+}
+
 static struct drgn_error *drgn_read_kdump(void *buf, uint64_t address,
 					  size_t count, uint64_t offset,
 					  void *arg, bool physical)
@@ -86,6 +130,7 @@ struct drgn_error *drgn_program_set_kdump(struct drgn_program *prog)
 	kdump_ctx_t *ctx;
 	kdump_status ks;
 	bool had_platform;
+	bool had_vmcoreinfo = prog->vmcoreinfo.raw;
 
 	ctx = kdump_new();
 	if (!ctx) {
@@ -93,6 +138,38 @@ struct drgn_error *drgn_program_set_kdump(struct drgn_program *prog)
 					 "kdump_new() failed");
 	}
 
+	/*
+	 * We need to be careful to set libkdumpfile attributes in the correct
+	 * order, in order to achive the desired result in the rare cases when
+	 * the program has the architecture and/or vmcoreinfo already set. In
+	 * these cases, frequently, the information is not available to
+	 * libkdumpfile, and so the user is providing it instead. We should set:
+	 *
+	 * - Architecture before file descriptor. This is because when
+	 *   libkdumpfile reads the vmcore, it immediately parses the notes in
+	 *   the header. If it doesn't know the current architecture, it will
+	 *   skip the architecture-specific notes, such as the PRSTATUS. This
+	 *   manifests to users as a missing "cpu.number" attribute, and thus a
+	 *   failure to get accurate stack traces for on-CPU tasks.
+	 * - Vmcoreinfo after architecture. This is because when libkdumpfile
+	 *   gets a new vmcoreinfo note set, it marks the address translation
+	 *   metadata as dirty and resets its address translation info using the
+	 *   new data. If no architecture is set, it skips this setup since it
+	 *   won't know how to do so. This manifests for users as FaultError
+	 *   when they access almost any memory address.
+	 *
+	 * For the common case, where we are just setting the FD and OS type,
+	 * and then letting libkdumpfile give us the vmcoreinfo and platform
+	 * info, the only ordering constraint is the obvious one: we need to set
+	 * the FD before getting data from libkdumpfile.
+	 */
+
+	if (prog->has_platform) {
+		err = drgn_platform_to_kdump(ctx, drgn_program_platform(prog));
+		if (err)
+			goto err;
+	}
+
 	ks = kdump_set_number_attr(ctx, KDUMP_ATTR_FILE_FD, prog->core_fd);
 	if (ks == KDUMP_ERR_NOTIMPL) {
 		err = drgn_error_format(DRGN_ERROR_INVALID_ARGUMENT,
@@ -113,30 +190,60 @@ struct drgn_error *drgn_program_set_kdump(struct drgn_program *prog)
 		goto err;
 	}
 
+	if (prog->vmcoreinfo.raw) {
 #if KDUMPFILE_VERSION >= KDUMPFILE_MKVER(0, 4, 1)
-	char *vmcoreinfo;
+		char *vmcoreinfo = memdup(prog->vmcoreinfo.raw, prog->vmcoreinfo.raw_size);
+		if (!vmcoreinfo) {
+			err = &drgn_enomem;
+			goto err;
+		}
+		kdump_blob_t *blob = kdump_blob_new(vmcoreinfo, prog->vmcoreinfo.raw_size);
+		if (!blob) {
+			free(vmcoreinfo);
+			err = &drgn_enomem;
+			goto err;
+		}
+		kdump_attr_t attr;
+		attr.type = KDUMP_BLOB;
+		attr.val.blob = blob;
+		ks = kdump_set_attr(ctx, "linux.vmcoreinfo.raw", &attr);
+		if (ks != KDUMP_OK) {
+			err = drgn_error_format(DRGN_ERROR_OTHER,
+						"kdump_set_attr(linux.vmcoreinfo.raw): %s",
+						kdump_get_err(ctx));
+			goto err;
+		}
 #else
-	const char *vmcoreinfo;
-#endif
-	ks = kdump_vmcoreinfo_raw(ctx, &vmcoreinfo);
-	if (ks != KDUMP_OK) {
-		err = drgn_error_format(DRGN_ERROR_OTHER,
-					"kdump_vmcoreinfo_raw: %s",
-					kdump_get_err(ctx));
+		err = drgn_error_create(DRGN_ERROR_NOT_IMPLEMENTED,
+					"overriding vmcoreinfo is not supported in libkdumpfile < 0.4.1");
 		goto err;
-	}
-
-	err = drgn_program_parse_vmcoreinfo(prog, vmcoreinfo,
-					    strlen(vmcoreinfo) + 1);
-	/*
-	 * As of libkdumpfile 0.4.1, the string returned by
-	 * kdump_vmcoreinfo_raw() needs to be freed.
-	 */
+#endif
+	} else {
 #if KDUMPFILE_VERSION >= KDUMPFILE_MKVER(0, 4, 1)
-	free(vmcoreinfo);
+		char *vmcoreinfo;
+#else
+		const char *vmcoreinfo;
 #endif
-	if (err)
-		goto err;
+		ks = kdump_vmcoreinfo_raw(ctx, &vmcoreinfo);
+		if (ks != KDUMP_OK) {
+			err = drgn_error_format(DRGN_ERROR_OTHER,
+						"kdump_vmcoreinfo_raw: %s",
+						kdump_get_err(ctx));
+			goto err;
+		}
+
+		err = drgn_program_parse_vmcoreinfo(prog, vmcoreinfo,
+						strlen(vmcoreinfo) + 1);
+		/*
+		* As of libkdumpfile 0.4.1, the string returned by
+		* kdump_vmcoreinfo_raw() needs to be freed.
+		*/
+#if KDUMPFILE_VERSION >= KDUMPFILE_MKVER(0, 4, 1)
+		free(vmcoreinfo);
+#endif
+		if (err)
+			goto err;
+	}
 
 	had_platform = prog->has_platform;
 	if (!had_platform) {
@@ -160,12 +267,9 @@ struct drgn_error *drgn_program_set_kdump(struct drgn_program *prog)
 	}
 
 	prog->flags |= DRGN_PROGRAM_IS_LINUX_KERNEL;
-	err = drgn_program_add_object_finder(prog, linux_kernel_object_find,
-					     prog);
+	err = drgn_program_finish_set_kernel(prog);
 	if (err)
 		goto err_platform;
-	if (!prog->lang)
-		prog->lang = &drgn_language_c;
 	prog->kdump_ctx = ctx;
 	return NULL;
 
@@ -173,8 +277,10 @@ err_platform:
 	prog->has_platform = had_platform;
 err:
 	// Reset anything we parsed from vmcoreinfo
-	free(prog->vmcoreinfo.raw);
-	memset(&prog->vmcoreinfo, 0, sizeof(prog->vmcoreinfo));
+	if (!had_vmcoreinfo) {
+		free(prog->vmcoreinfo.raw);
+		memset(&prog->vmcoreinfo, 0, sizeof(prog->vmcoreinfo));
+	}
 	kdump_free(ctx);
 	return err;
 }
