@@ -12,6 +12,7 @@
 
 #include "array.h"
 #include "bitops.h"
+#include "c_lexer.h"
 #include "error.h"
 #include "language.h" // IWYU pragma: associated
 #include "lexer.h"
@@ -1261,9 +1262,8 @@ c_format_pointer_object(const struct drgn_object *obj,
 	bool c_string =
 		((flags & DRGN_FORMAT_OBJECT_STRING) &&
 		 is_character_type(drgn_type_type(underlying_type).type));
-	bool have_symbol;
 	uint64_t uvalue;
-	struct drgn_symbol sym;
+	_cleanup_symbol_ struct drgn_symbol *sym = NULL;
 	size_t start, type_start, type_end, value_start, value_end;
 
 	start = sb->len;
@@ -1287,18 +1287,17 @@ c_format_pointer_object(const struct drgn_object *obj,
 	if (err)
 		return err;
 
-	have_symbol = ((flags & DRGN_FORMAT_OBJECT_SYMBOLIZE) &&
-		       drgn_program_find_symbol_by_address_internal(drgn_object_program(obj),
-								    uvalue,
-								    NULL,
-								    &sym));
-	if (have_symbol && dereference && !c_string &&
+	if ((flags & DRGN_FORMAT_OBJECT_SYMBOLIZE) &&
+	    (err = drgn_program_find_symbol_by_address_internal(drgn_object_program(obj),
+								uvalue, &sym)))
+		return err;
+	if (sym && dereference && !c_string &&
 	    !string_builder_appendc(sb, '('))
 		return &drgn_enomem;
 	value_start = sb->len;
-	if (have_symbol &&
-	     !string_builder_appendf(sb, "%s+0x%" PRIx64 " = ", sym.name,
-				     uvalue - sym.address))
+	if (sym &&
+	     !string_builder_appendf(sb, "%s+0x%" PRIx64 " = ", sym->name,
+				     uvalue - sym->address))
 		return &drgn_enomem;
 
 	if (!string_builder_appendf(sb, "0x%" PRIx64, uvalue))
@@ -1307,7 +1306,7 @@ c_format_pointer_object(const struct drgn_object *obj,
 		return NULL;
 	value_end = sb->len;
 
-	if ((have_symbol && dereference && !c_string &&
+	if ((sym && dereference && !c_string &&
 	     !string_builder_appendc(sb, ')')) ||
 	    !string_builder_append(sb, " = "))
 		return &drgn_enomem;
@@ -1613,53 +1612,8 @@ static struct drgn_error *c_format_object(const struct drgn_object *obj,
 	return NULL;
 }
 
-/* This obviously incomplete since we only handle the tokens we care about. */
-enum {
-	C_TOKEN_EOF = -1,
-	MIN_KEYWORD_TOKEN,
-	MIN_SPECIFIER_TOKEN = MIN_KEYWORD_TOKEN,
-	C_TOKEN_VOID = MIN_SPECIFIER_TOKEN,
-	C_TOKEN_CHAR,
-	C_TOKEN_SHORT,
-	C_TOKEN_INT,
-	C_TOKEN_LONG,
-	C_TOKEN_SIGNED,
-	C_TOKEN_UNSIGNED,
-	C_TOKEN_BOOL,
-	C_TOKEN_FLOAT,
-	C_TOKEN_DOUBLE,
-	C_TOKEN_COMPLEX,
-	MAX_SPECIFIER_TOKEN = C_TOKEN_COMPLEX,
-	MIN_QUALIFIER_TOKEN,
-	C_TOKEN_CONST = MIN_QUALIFIER_TOKEN,
-	C_TOKEN_RESTRICT,
-	C_TOKEN_VOLATILE,
-	C_TOKEN_ATOMIC,
-	MAX_QUALIFIER_TOKEN = C_TOKEN_ATOMIC,
-	C_TOKEN_STRUCT,
-	C_TOKEN_UNION,
-	C_TOKEN_CLASS,
-	C_TOKEN_ENUM,
-	MAX_KEYWORD_TOKEN = C_TOKEN_ENUM,
-	C_TOKEN_LPAREN,
-	C_TOKEN_RPAREN,
-	C_TOKEN_LBRACKET,
-	C_TOKEN_RBRACKET,
-	C_TOKEN_ASTERISK,
-	C_TOKEN_DOT,
-	C_TOKEN_NUMBER,
-	C_TOKEN_IDENTIFIER,
-	C_TOKEN_TEMPLATE_ARGUMENTS,
-	C_TOKEN_COLON,
-};
-
 #include "c_keywords.inc"
 
-struct drgn_c_family_lexer {
-	struct drgn_lexer lexer;
-	bool cpp;
-};
-
 struct drgn_error *drgn_c_family_lexer_func(struct drgn_lexer *lexer,
 					    struct drgn_token *token) {
 	const char *p = lexer->p;
