@@ -8,6 +8,8 @@
 
 struct drgn_debug_info_load_state;
 
+struct drgn_error *drgn_program_finish_set_kernel(struct drgn_program *prog);
+
 struct drgn_error *read_memory_via_pgtable(void *buf, uint64_t address,
 					   size_t count, uint64_t offset,
 					   void *arg, bool physical);
@@ -21,11 +23,6 @@ struct drgn_error *proc_kallsyms_symbol_addr(const char *name,
 
 struct drgn_error *read_vmcoreinfo_fallback(struct drgn_program *prog);
 
-struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
-					    const char *filename,
-					    enum drgn_find_object_flags flags,
-					    void *arg, struct drgn_object *ret);
-
 struct drgn_error *
 linux_kernel_report_debug_info(struct drgn_debug_info_load_state *load);
 
