@@ -28,7 +28,6 @@
 #include "linux_kernel.h"
 #include "memory_reader.h"
 #include "minmax.h"
-#include "object_index.h"
 #include "program.h"
 #include "symbol.h"
 #include "util.h"
@@ -39,7 +38,6 @@ static inline uint32_t drgn_thread_to_key(const struct drgn_thread *entry)
 	return entry->tid;
 }
 
-DEFINE_VECTOR_FUNCTIONS(drgn_prstatus_vector);
 DEFINE_HASH_TABLE_FUNCTIONS(drgn_thread_set, drgn_thread_to_key,
 			    int_key_hash_pair, scalar_key_eq);
 
@@ -100,11 +98,11 @@ void drgn_program_init(struct drgn_program *prog,
 	memset(prog, 0, sizeof(*prog));
 	drgn_memory_reader_init(&prog->reader);
 	drgn_program_init_types(prog);
-	drgn_object_index_init(&prog->oindex);
 	drgn_debug_info_init(&prog->dbinfo, prog);
 	prog->core_fd = -1;
 	if (platform)
 		drgn_program_set_platform(prog, platform);
+	drgn_thread_set_init(&prog->thread_set);
 	char *env = getenv("DRGN_PREFER_ORC_UNWINDER");
 	prog->prefer_orc_unwinder = env && atoi(env);
 	drgn_program_set_log_level(prog, DRGN_LOG_NONE);
@@ -114,12 +112,7 @@ void drgn_program_init(struct drgn_program *prog,
 
 void drgn_program_deinit(struct drgn_program *prog)
 {
-	if (prog->core_dump_notes_cached) {
-		if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
-			drgn_prstatus_vector_deinit(&prog->prstatus_vector);
-		else
-			drgn_thread_set_deinit(&prog->thread_set);
-	}
+	drgn_thread_set_deinit(&prog->thread_set);
 	/*
 	 * For userspace core dumps, main_thread and crashed_thread are in
 	 * prog->thread_set and thus freed by the above call to
@@ -134,7 +127,16 @@ void drgn_program_deinit(struct drgn_program *prog)
 
 	drgn_object_deinit(&prog->vmemmap);
 
-	drgn_object_index_deinit(&prog->oindex);
+	drgn_handler_list_deinit(struct drgn_symbol_finder, finder,
+				 &prog->symbol_finders,
+		if (finder->ops.destroy)
+			finder->ops.destroy(finder->arg);
+	);
+	drgn_handler_list_deinit(struct drgn_object_finder, finder,
+				 &prog->object_finders,
+		if (finder->ops.destroy)
+			finder->ops.destroy(finder->arg);
+	);
 	drgn_program_deinit_types(prog);
 	drgn_memory_reader_deinit(&prog->reader);
 
@@ -191,20 +193,82 @@ drgn_program_add_memory_segment(struct drgn_program *prog, uint64_t address,
 					      physical);
 }
 
-struct drgn_error *
-drgn_program_add_object_finder_impl(struct drgn_program *prog,
-				    struct drgn_object_finder *finder,
-				    drgn_object_find_fn fn, void *arg)
-{
-	return drgn_object_index_add_finder(&prog->oindex, finder, fn, arg);
+#define DRGN_PROGRAM_FINDER(which)						\
+struct drgn_error *								\
+drgn_program_register_##which##_finder_impl(struct drgn_program *prog,		\
+					    struct drgn_##which##_finder *finder,\
+					    const char *name,			\
+					    const struct drgn_##which##_finder_ops *ops,\
+					    void *arg, size_t enable_index)	\
+{										\
+	struct drgn_error *err;							\
+	if (finder) {								\
+		finder->handler.name = name;					\
+		finder->handler.free = false;					\
+	} else {								\
+		finder = malloc(sizeof(*finder));				\
+		if (!finder)							\
+			return &drgn_enomem;					\
+		finder->handler.name = strdup(name);				\
+		if (!finder->handler.name) {					\
+			free(finder);						\
+			return &drgn_enomem;					\
+		}								\
+		finder->handler.free = true;					\
+	}									\
+	memcpy(&finder->ops, ops, sizeof(finder->ops));				\
+	finder->arg = arg;							\
+	err = drgn_handler_list_register(&prog->which##_finders,		\
+					 &finder->handler, enable_index,	\
+					 #which " finder");			\
+	if (err && finder->handler.free) {					\
+		free((char *)finder->handler.name);				\
+		free(finder);							\
+	}									\
+	return err;								\
+}										\
+										\
+LIBDRGN_PUBLIC struct drgn_error *						\
+drgn_program_register_##which##_finder(struct drgn_program *prog, const char *name,\
+				       const struct drgn_##which##_finder_ops *ops,\
+				       void *arg, size_t enable_index)		\
+{										\
+	return drgn_program_register_##which##_finder_impl(prog, NULL, name,	\
+							   ops, arg,		\
+							   enable_index);	\
+}										\
+										\
+LIBDRGN_PUBLIC struct drgn_error *						\
+drgn_program_registered_##which##_finders(struct drgn_program *prog,		\
+					  const char ***names_ret,		\
+					  size_t *count_ret)			\
+{										\
+	return drgn_handler_list_registered(&prog->which##_finders, names_ret,	\
+					    count_ret);				\
+}										\
+										\
+LIBDRGN_PUBLIC struct drgn_error *						\
+drgn_program_set_enabled_##which##_finders(struct drgn_program *prog,		\
+					   const char * const *names,		\
+					   size_t count)			\
+{										\
+	return drgn_handler_list_set_enabled(&prog->which##_finders, names,	\
+					     count, #which "finder");		\
+}										\
+										\
+LIBDRGN_PUBLIC struct drgn_error *						\
+drgn_program_enabled_##which##_finders(struct drgn_program *prog,		\
+				       const char ***names_ret,			\
+				       size_t *count_ret)			\
+{										\
+	return drgn_handler_list_enabled(&prog->which##_finders, names_ret,	\
+					 count_ret);				\
 }
 
-LIBDRGN_PUBLIC struct drgn_error *
-drgn_program_add_object_finder(struct drgn_program *prog,
-			       drgn_object_find_fn fn, void *arg)
-{
-	return drgn_program_add_object_finder_impl(prog, NULL, fn, arg);
-}
+DRGN_PROGRAM_FINDER(type)
+DRGN_PROGRAM_FINDER(object)
+DRGN_PROGRAM_FINDER(symbol)
+#undef DRGN_PROGRAM_FINDER
 
 static struct drgn_error *
 drgn_program_check_initialized(struct drgn_program *prog)
@@ -252,6 +316,8 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 	const char *vmcoreinfo_note = NULL;
 	size_t vmcoreinfo_size = 0;
 	bool have_nt_taskstruct = false, is_proc_kcore;
+	bool have_vmcoreinfo = prog->vmcoreinfo.raw;
+	bool had_vmcoreinfo = have_vmcoreinfo;
 
 	prog->core_fd = fd;
 	err = has_kdump_signature(prog, path, &is_kdump);
@@ -361,6 +427,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 					 * may be valid.
 					 */
 					have_phys_addrs = true;
+					have_vmcoreinfo = true;
 				} else if (nhdr.n_namesz == sizeof("QEMU") &&
 					   memcmp(name, "QEMU",
 						  sizeof("QEMU")) == 0) {
@@ -387,7 +454,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 		is_proc_kcore = false;
 	}
 
-	if (vmcoreinfo_note && !is_proc_kcore) {
+	if (have_vmcoreinfo && !is_proc_kcore) {
 		char *env;
 
 		/* Use libkdumpfile for ELF vmcores if it was requested. */
@@ -408,7 +475,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 	}
 
 	bool pgtable_reader =
-		(is_proc_kcore || vmcoreinfo_note) &&
+		(is_proc_kcore || have_vmcoreinfo) &&
 		prog->platform.arch->linux_kernel_pgtable_iterator_next;
 	if (pgtable_reader) {
 		/*
@@ -469,7 +536,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 		 * all zeroes and memory that was excluded by makedumpfile for
 		 * another reason, so we're forced to always return zeroes.
 		 */
-		prog->file_segments[j].zerofill = vmcoreinfo_note && !is_proc_kcore;
+		prog->file_segments[j].zerofill = have_vmcoreinfo && !is_proc_kcore;
 		err = drgn_program_add_memory_segment(prog, phdr->p_vaddr,
 						      phdr->p_memsz,
 						      drgn_read_memory_file,
@@ -538,7 +605,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 			j++;
 		}
 	}
-	if (vmcoreinfo_note) {
+	if (vmcoreinfo_note && !prog->vmcoreinfo.raw) {
 		err = drgn_program_parse_vmcoreinfo(prog, vmcoreinfo_note,
 						    vmcoreinfo_size);
 		if (err)
@@ -546,7 +613,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 	}
 
 	if (is_proc_kcore) {
-		if (!vmcoreinfo_note) {
+		if (!have_vmcoreinfo) {
 			err = read_vmcoreinfo_fallback(prog);
 			if (err)
 				goto out_segments;
@@ -556,7 +623,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 		                DRGN_PROGRAM_IS_LOCAL);
 		elf_end(prog->core);
 		prog->core = NULL;
-	} else if (vmcoreinfo_note) {
+	} else if (have_vmcoreinfo) {
 		prog->flags |= DRGN_PROGRAM_IS_LINUX_KERNEL;
 	} else if (have_qemu_note) {
 		err = drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
@@ -569,13 +636,9 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 		goto out_segments;
 	}
 	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
-		err = drgn_program_add_object_finder(prog,
-						     linux_kernel_object_find,
-						     prog);
+		err = drgn_program_finish_set_kernel(prog);
 		if (err)
 			goto out_segments;
-		if (!prog->lang)
-			prog->lang = &drgn_language_c;
 	}
 
 	return NULL;
@@ -588,8 +651,11 @@ out_segments:
 out_notes:
 	// Reset anything we parsed from ELF notes.
 	prog->aarch64_insn_pac_mask = 0;
-	free(prog->vmcoreinfo.raw);
-	memset(&prog->vmcoreinfo, 0, sizeof(prog->vmcoreinfo));
+	// Free vmcoreinfo buffer if it was not provided by the caller
+	if (!had_vmcoreinfo) {
+		free(prog->vmcoreinfo.raw);
+		memset(&prog->vmcoreinfo, 0, sizeof(prog->vmcoreinfo));
+	}
 out_platform:
 	prog->has_platform = had_platform;
 out_elf:
@@ -858,27 +924,17 @@ struct drgn_error *drgn_program_cache_prstatus_entry(struct drgn_program *prog,
 						     const char *data,
 						     size_t size, uint32_t *ret)
 {
-	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
-		struct nstring *entry =
-			drgn_prstatus_vector_append_entry(&prog->prstatus_vector);
-		if (!entry)
-			return &drgn_enomem;
-		entry->str = data;
-		entry->len = size;
-	} else {
-		struct drgn_thread thread = {
-			.prog = prog,
-			.prstatus = { data, size },
-	       };
-		struct drgn_error *err = get_prstatus_pid(prog, data, size,
-							  &thread.tid);
-		if (err)
-			return err;
-		*ret = thread.tid;
-		if (drgn_thread_set_insert(&prog->thread_set, &thread,
-					   NULL) == -1)
-			return &drgn_enomem;
-	}
+	struct drgn_thread thread = {
+		.prog = prog,
+		.prstatus = { data, size },
+	};
+	struct drgn_error *err = get_prstatus_pid(prog, data, size,
+						  &thread.tid);
+	if (err)
+		return err;
+	*ret = thread.tid;
+	if (drgn_thread_set_insert(&prog->thread_set, &thread, NULL) == -1)
+		return &drgn_enomem;
 	return NULL;
 }
 
@@ -897,11 +953,6 @@ drgn_program_cache_core_dump_notes(struct drgn_program *prog)
 
 	assert(!(prog->flags & DRGN_PROGRAM_IS_LIVE));
 
-	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
-		drgn_prstatus_vector_init(&prog->prstatus_vector);
-	else
-		drgn_thread_set_init(&prog->thread_set);
-
 #ifdef WITH_LIBKDUMPFILE
 	if (prog->kdump_ctx) {
 		err = drgn_program_cache_kdump_notes(prog);
@@ -1005,10 +1056,8 @@ out:
 	return NULL;
 
 err:
-	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
-		drgn_prstatus_vector_deinit(&prog->prstatus_vector);
-	else
-		drgn_thread_set_deinit(&prog->thread_set);
+	drgn_thread_set_deinit(&prog->thread_set);
+	drgn_thread_set_init(&prog->thread_set);
 	return err;
 }
 
@@ -1337,6 +1386,7 @@ drgn_program_find_thread_kernel_cpu_curr(struct drgn_program *prog,
 	if (err)
 		goto out;
 	thread->tid = tid.uvalue;
+	thread->prstatus = (struct nstring){};
 
 	*ret = thread;
 
@@ -1355,7 +1405,6 @@ drgn_program_kernel_core_dump_cache_crashed_thread(struct drgn_program *prog)
 
 	assert((prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) &&
 	       !(prog->flags & DRGN_PROGRAM_IS_LIVE));
-	assert(prog->core_dump_notes_cached);
 	if (prog->crashed_thread)
 		return NULL;
 
@@ -1364,17 +1413,12 @@ drgn_program_kernel_core_dump_cache_crashed_thread(struct drgn_program *prog)
 	if (err)
 		return err;
 
-	if (crashed_cpu >= drgn_prstatus_vector_size(&prog->prstatus_vector))
-		return NULL;
-
 	err = drgn_program_find_thread_kernel_cpu_curr(prog, crashed_cpu,
 						       &prog->crashed_thread);
 	if (err) {
 		prog->crashed_thread = NULL;
 		return err;
 	}
-	prog->crashed_thread->prstatus =
-		*drgn_prstatus_vector_at(&prog->prstatus_vector, crashed_cpu);
 	return NULL;
 }
 
@@ -1418,14 +1462,12 @@ drgn_program_crashed_thread(struct drgn_program *prog, struct drgn_thread **ret)
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "crashed thread is only defined for core dumps");
 	}
-	err = drgn_program_cache_core_dump_notes(prog);
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		err = drgn_program_kernel_core_dump_cache_crashed_thread(prog);
+	else
+		err = drgn_program_cache_core_dump_notes(prog);
 	if (err)
 		return err;
-	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
-		err = drgn_program_kernel_core_dump_cache_crashed_thread(prog);
-		if (err)
-			return err;
-	}
 	if (!prog->crashed_thread) {
 		return drgn_error_create(DRGN_ERROR_OTHER,
 					 "crashed thread not found");
@@ -1445,36 +1487,14 @@ drgn_thread_object(struct drgn_thread *thread, const struct drgn_object **ret)
 	return NULL;
 }
 
-struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
-						     uint32_t cpu,
-						     struct nstring *ret)
+struct drgn_error *drgn_program_find_prstatus(struct drgn_program *prog,
+					      uint32_t tid, struct nstring *ret)
 {
-	assert(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL);
 	struct drgn_error *err = drgn_program_cache_core_dump_notes(prog);
 	if (err)
 		return err;
-
-	if (cpu < drgn_prstatus_vector_size(&prog->prstatus_vector)) {
-		*ret = *drgn_prstatus_vector_at(&prog->prstatus_vector, cpu);
-	} else {
-		ret->str = NULL;
-		ret->len = 0;
-	}
-	return NULL;
-}
-
-struct drgn_error *drgn_program_find_prstatus_by_tid(struct drgn_program *prog,
-						     uint32_t tid,
-						     struct nstring *ret)
-{
-	struct drgn_error *err;
-
-	assert(!(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL));
-	struct drgn_thread *thread;
-	err = drgn_program_find_thread(prog, tid, &thread);
-	if (err)
-		return err;
-
+	struct drgn_thread *thread =
+		drgn_thread_set_search(&prog->thread_set, &tid).entry;
 	if (!thread) {
 		ret->str = NULL;
 		ret->len = 0;
@@ -1762,33 +1782,50 @@ drgn_program_find_object(struct drgn_program *prog, const char *name,
 			 enum drgn_find_object_flags flags,
 			 struct drgn_object *ret)
 {
+	struct drgn_error *err;
+
+	if ((flags & ~DRGN_FIND_OBJECT_ANY) || !flags) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "invalid find object flags");
+	}
 	if (ret && drgn_object_program(ret) != prog) {
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "object is from wrong program");
 	}
-	return drgn_object_index_find(&prog->oindex, name, filename, flags,
-				      ret);
-}
 
-bool drgn_program_find_symbol_by_address_internal(struct drgn_program *prog,
-						  uint64_t address,
-						  Dwfl_Module *module,
-						  struct drgn_symbol *ret)
-{
-	if (!module) {
-		module = dwfl_addrmodule(prog->dbinfo.dwfl, address);
-		if (!module)
-			return false;
+	size_t name_len = strlen(name);
+	drgn_handler_list_for_each_enabled(struct drgn_object_finder, finder,
+					   &prog->object_finders) {
+		err = finder->ops.find(name, name_len, filename, flags,
+				       finder->arg, ret);
+		if (err != &drgn_not_found)
+			return err;
 	}
 
-	GElf_Off offset;
-	GElf_Sym elf_sym;
-	const char *name = dwfl_module_addrinfo(module, address, &offset,
-						&elf_sym, NULL, NULL, NULL);
-	if (!name)
-		return false;
-	drgn_symbol_from_elf(name, address - offset, &elf_sym, ret);
-	return true;
+	const char *kind_str;
+	switch (flags) {
+	case DRGN_FIND_OBJECT_CONSTANT:
+		kind_str = "constant ";
+		break;
+	case DRGN_FIND_OBJECT_FUNCTION:
+		kind_str = "function ";
+		break;
+	case DRGN_FIND_OBJECT_VARIABLE:
+		kind_str = "variable ";
+		break;
+	default:
+		kind_str = "";
+		break;
+	}
+	if (filename) {
+		return drgn_error_format(DRGN_ERROR_LOOKUP,
+					 "could not find %s'%s' in '%s'",
+					 kind_str, name, filename);
+	} else {
+		return drgn_error_format(DRGN_ERROR_LOOKUP,
+					 "could not find %s'%s'", kind_str,
+					 name);
+	}
 }
 
 struct drgn_error *drgn_error_symbol_not_found(uint64_t address)
@@ -1798,115 +1835,19 @@ struct drgn_error *drgn_error_symbol_not_found(uint64_t address)
 				 address);
 }
 
-LIBDRGN_PUBLIC struct drgn_error *
-drgn_program_find_symbol_by_address(struct drgn_program *prog, uint64_t address,
-				    struct drgn_symbol **ret)
-{
-	struct drgn_symbol *sym;
-
-	sym = malloc(sizeof(*sym));
-	if (!sym)
-		return &drgn_enomem;
-	if (!drgn_program_find_symbol_by_address_internal(prog, address, NULL,
-							  sym)) {
-		free(sym);
-		return drgn_error_symbol_not_found(address);
-	}
-	*ret = sym;
-	return NULL;
-}
-
-DEFINE_VECTOR(symbolp_vector, struct drgn_symbol *);
-
-enum {
-	SYMBOLS_SEARCH_NAME = (1 << 0),
-	SYMBOLS_SEARCH_ADDRESS = (1 << 1),
-	SYMBOLS_SEARCH_ALL = (1 << 2),
-};
-
-struct symbols_search_arg {
-	const char *name;
-	uint64_t address;
-	struct symbolp_vector results;
-	unsigned int flags;
-};
-
-static bool symbol_match(struct symbols_search_arg *arg, GElf_Addr addr,
-			 const GElf_Sym *sym, const char *name)
-{
-	if (arg->flags & SYMBOLS_SEARCH_ALL)
-		return true;
-	if ((arg->flags & SYMBOLS_SEARCH_NAME) && strcmp(name, arg->name) == 0)
-		return true;
-	if ((arg->flags & SYMBOLS_SEARCH_ADDRESS) &&
-	    arg->address >= addr && arg->address < addr + sym->st_size)
-		return true;
-	return false;
-}
-
-static int symbols_search_cb(Dwfl_Module *dwfl_module, void **userdatap,
-			     const char *module_name, Dwarf_Addr base,
-			     void *cb_arg)
-{
-	struct symbols_search_arg *arg = cb_arg;
-
-	int symtab_len = dwfl_module_getsymtab(dwfl_module);
-	if (symtab_len == -1)
-		return DWARF_CB_OK;
-
-	/* Ignore the zeroth null symbol */
-	for (int i = 1; i < symtab_len; i++) {
-		GElf_Sym elf_sym;
-		GElf_Addr elf_addr;
-		const char *name = dwfl_module_getsym_info(dwfl_module, i,
-							   &elf_sym, &elf_addr,
-							   NULL, NULL, NULL);
-		if (!name || !symbol_match(arg, elf_addr, &elf_sym, name))
-			continue;
-
-		struct drgn_symbol *sym = malloc(sizeof(*sym));
-		if (!sym)
-			return DWARF_CB_ABORT;
-		drgn_symbol_from_elf(name, elf_addr, &elf_sym, sym);
-		if (!symbolp_vector_append(&arg->results, &sym)) {
-			drgn_symbol_destroy(sym);
-			return DWARF_CB_ABORT;
-		}
-	}
-	return DWARF_CB_OK;
-}
-
 static struct drgn_error *
-symbols_search(struct drgn_program *prog, struct symbols_search_arg *arg,
-	       struct drgn_symbol ***syms_ret, size_t *count_ret)
+drgn_program_symbols_search(struct drgn_program *prog, const char *name,
+			    uint64_t addr, enum drgn_find_symbol_flags flags,
+			    struct drgn_symbol_result_builder *builder)
 {
-	struct drgn_error *err;
-
-	symbolp_vector_init(&arg->results);
-
-	/*
-	 * When searching for addresses, we can identify the exact module to
-	 * search. Otherwise we need to fall back to an exhaustive search.
-	 */
-	err = NULL;
-	if (arg->flags & SYMBOLS_SEARCH_ADDRESS) {
-		Dwfl_Module *module = dwfl_addrmodule(prog->dbinfo.dwfl,
-						      arg->address);
-		if (module && symbols_search_cb(module, NULL, NULL, 0, arg))
-			err = &drgn_enomem;
-	} else {
-		if (dwfl_getmodules(prog->dbinfo.dwfl, symbols_search_cb, arg,
-				    0))
-			err = &drgn_enomem;
-	}
-
-	if (err) {
-		vector_for_each(symbolp_vector, symbolp, &arg->results)
-			drgn_symbol_destroy(*symbolp);
-		symbolp_vector_deinit(&arg->results);
-	} else {
-		symbolp_vector_shrink_to_fit(&arg->results);
-		symbolp_vector_steal(&arg->results, syms_ret, count_ret);
+	struct drgn_error *err = NULL;
+	drgn_handler_list_for_each_enabled(struct drgn_symbol_finder, finder,
+					   &prog->symbol_finders) {
+		err = finder->ops.find(name, addr, flags, finder->arg, builder);
+		if (err ||
+		    ((flags & DRGN_FIND_SYMBOL_ONE)
+		     && drgn_symbol_result_builder_count(builder) > 0))
+			break;
 	}
 	return err;
 }
@@ -1916,11 +1857,17 @@ drgn_program_find_symbols_by_name(struct drgn_program *prog, const char *name,
 				  struct drgn_symbol ***syms_ret,
 				  size_t *count_ret)
 {
-	struct symbols_search_arg arg = {
-		.name = name,
-		.flags = name ? SYMBOLS_SEARCH_NAME : SYMBOLS_SEARCH_ALL,
-	};
-	return symbols_search(prog, &arg, syms_ret, count_ret);
+	struct drgn_symbol_result_builder builder;
+	enum drgn_find_symbol_flags flags = name ? DRGN_FIND_SYMBOL_NAME : 0;
+
+	drgn_symbol_result_builder_init(&builder, false);
+	struct drgn_error *err = drgn_program_symbols_search(prog, name, 0,
+							     flags, &builder);
+	if (err)
+		drgn_symbol_result_builder_abort(&builder);
+	else
+		drgn_symbol_result_builder_array(&builder, syms_ret, count_ret);
+	return err;
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
@@ -1929,88 +1876,81 @@ drgn_program_find_symbols_by_address(struct drgn_program *prog,
 				     struct drgn_symbol ***syms_ret,
 				     size_t *count_ret)
 {
-	struct symbols_search_arg arg = {
-		.address = address,
-		.flags = SYMBOLS_SEARCH_ADDRESS,
-	};
-	return symbols_search(prog, &arg, syms_ret, count_ret);
-}
+	struct drgn_symbol_result_builder builder;
+	enum drgn_find_symbol_flags flags = DRGN_FIND_SYMBOL_ADDR;
 
-struct find_symbol_by_name_arg {
-	const char *name;
-	GElf_Sym sym;
-	GElf_Addr addr;
-	bool found;
-	bool bad_symtabs;
-};
-
-static int find_symbol_by_name_cb(Dwfl_Module *dwfl_module, void **userdatap,
-				  const char *module_name, Dwarf_Addr base,
-				  void *cb_arg)
-{
-	struct find_symbol_by_name_arg *arg = cb_arg;
-	int symtab_len = dwfl_module_getsymtab(dwfl_module);
-	if (symtab_len == -1) {
-		arg->bad_symtabs = true;
-		return DWARF_CB_OK;
-	}
-	/*
-	 * Global symbols are after local symbols, so by iterating backwards we
-	 * might find a global symbol faster. Ignore the zeroth null symbol.
-	 */
-	for (int i = symtab_len - 1; i > 0; i--) {
-		GElf_Sym sym;
-		GElf_Addr addr;
-		const char *name = dwfl_module_getsym_info(dwfl_module, i, &sym,
-							   &addr, NULL, NULL,
-							   NULL);
-		if (name && strcmp(arg->name, name) == 0) {
-			/*
-			 * The order of precedence is
-			 * GLOBAL = GNU_UNIQUE > WEAK > LOCAL = everything else
-			 *
-			 * If we found a global or unique symbol, return it
-			 * immediately. If we found a weak symbol, then save it,
-			 * which may overwrite a previously found weak or local
-			 * symbol. Otherwise, save the symbol only if we haven't
-			 * found another symbol.
-			 */
-			if (GELF_ST_BIND(sym.st_info) == STB_GLOBAL ||
-			    GELF_ST_BIND(sym.st_info) == STB_GNU_UNIQUE ||
-			    GELF_ST_BIND(sym.st_info) == STB_WEAK ||
-			    !arg->found) {
-				arg->sym = sym;
-				arg->addr = addr;
-				arg->found = true;
-			}
-			if (GELF_ST_BIND(sym.st_info) == STB_GLOBAL ||
-			    GELF_ST_BIND(sym.st_info) == STB_GNU_UNIQUE)
-				return DWARF_CB_ABORT;
-		}
-	}
-	return DWARF_CB_OK;
+	drgn_symbol_result_builder_init(&builder, false);
+	struct drgn_error *err = drgn_program_symbols_search(prog, NULL, address,
+							     flags, &builder);
+	if (err)
+		drgn_symbol_result_builder_abort(&builder);
+	else
+		drgn_symbol_result_builder_array(&builder, syms_ret, count_ret);
+	return err;
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_find_symbol_by_name(struct drgn_program *prog,
-			const char *name, struct drgn_symbol **ret)
+				 const char *name, struct drgn_symbol **ret)
 {
-	struct find_symbol_by_name_arg arg = {
-		.name = name,
-	};
-	dwfl_getmodules(prog->dbinfo.dwfl, find_symbol_by_name_cb, &arg, 0);
-	if (arg.found) {
-		struct drgn_symbol *sym = malloc(sizeof(*sym));
-		if (!sym)
-			return &drgn_enomem;
-		drgn_symbol_from_elf(name, arg.addr, &arg.sym, sym);
-		*ret = sym;
-		return NULL;
+	struct drgn_symbol_result_builder builder;
+	enum drgn_find_symbol_flags flags = DRGN_FIND_SYMBOL_NAME | DRGN_FIND_SYMBOL_ONE;
+
+	drgn_symbol_result_builder_init(&builder, true);
+	struct drgn_error *err = drgn_program_symbols_search(prog, name, 0,
+							     flags, &builder);
+	if (err) {
+		drgn_symbol_result_builder_abort(&builder);
+		return err;
 	}
-	return drgn_error_format(DRGN_ERROR_LOOKUP,
-				 "could not find symbol with name '%s'%s", name,
-				 arg.bad_symtabs ?
-				 " (could not get some symbol tables)" : "");
+
+	if (!drgn_symbol_result_builder_count(&builder))
+		return drgn_error_format(DRGN_ERROR_LOOKUP,
+					 "could not find symbol with name '%s'", name);
+
+	*ret = drgn_symbol_result_builder_single(&builder);
+	return err;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_find_symbol_by_address(struct drgn_program *prog, uint64_t address,
+				    struct drgn_symbol **ret)
+{
+	struct drgn_symbol_result_builder builder;
+	enum drgn_find_symbol_flags flags = DRGN_FIND_SYMBOL_ADDR | DRGN_FIND_SYMBOL_ONE;
+
+	drgn_symbol_result_builder_init(&builder, true);
+	struct drgn_error *err = drgn_program_symbols_search(prog, NULL, address,
+							     flags, &builder);
+
+	if (err) {
+		drgn_symbol_result_builder_abort(&builder);
+		return err;
+	}
+
+	if (!drgn_symbol_result_builder_count(&builder))
+		return drgn_error_symbol_not_found(address);
+
+	*ret = drgn_symbol_result_builder_single(&builder);
+	return err;
+}
+
+struct drgn_error *
+drgn_program_find_symbol_by_address_internal(struct drgn_program *prog,
+					     uint64_t address,
+					     struct drgn_symbol **ret)
+{
+	struct drgn_symbol_result_builder builder;
+	enum drgn_find_symbol_flags flags = DRGN_FIND_SYMBOL_ADDR | DRGN_FIND_SYMBOL_ONE;
+
+	drgn_symbol_result_builder_init(&builder, true);
+	struct drgn_error *err = drgn_program_symbols_search(prog, NULL, address,
+							     flags, &builder);
+	if (err)
+		drgn_symbol_result_builder_abort(&builder);
+	else
+		*ret = drgn_symbol_result_builder_single(&builder);
+	return err;
 }
 
 LIBDRGN_PUBLIC struct drgn_error *
