@@ -21,15 +21,20 @@
 
 #include "debug_info.h"
 #include "drgn.h"
+#include "handler.h"
 #include "hash_table.h"
 #include "language.h"
 #include "memory_reader.h"
-#include "object_index.h"
 #include "platform.h"
 #include "pp.h"
+#include "symbol.h"
 #include "type.h"
 #include "vector.h"
 
+struct drgn_type_finder;
+struct drgn_object_finder;
+struct drgn_symbol_finder;
+
 /**
  * @defgroup Internals Internals
  *
@@ -52,7 +57,6 @@ struct drgn_thread {
 };
 
 DEFINE_VECTOR_TYPE(drgn_typep_vector, struct drgn_type *);
-DEFINE_VECTOR_TYPE(drgn_prstatus_vector, struct nstring);
 DEFINE_HASH_TABLE_TYPE(drgn_thread_set, struct drgn_thread);
 
 struct drgn_program {
@@ -78,7 +82,7 @@ struct drgn_program {
 	 * Types.
 	 */
 	/** Callbacks for finding types. */
-	struct drgn_type_finder *type_finders;
+	struct drgn_handler_list type_finders;
 	/** Void type for each language. */
 	struct drgn_type void_types[DRGN_NUM_LANGUAGES];
 	/** Cache of primitive types. */
@@ -107,8 +111,9 @@ struct drgn_program {
 	/*
 	 * Debugging information.
 	 */
-	struct drgn_object_index oindex;
+	struct drgn_handler_list object_finders;
 	struct drgn_debug_info dbinfo;
+	struct drgn_handler_list symbol_finders;
 
 	/*
 	 * Program information.
@@ -122,16 +127,13 @@ struct drgn_program {
 	/*
 	 * Threads/stack traces.
 	 */
-	union {
-		/*
-		 * For the Linux kernel, PRSTATUS notes indexed by CPU. See
-		 * drgn_get_initial_registers() for why we don't use the PID
-		 * map.
-		 */
-		struct drgn_prstatus_vector prstatus_vector;
-		/* For userspace programs, threads indexed by PID. */
-		struct drgn_thread_set thread_set;
-	};
+	/*
+	 * Threads indexed by TID.
+	 *
+	 * For the Linux kernel, this is only used to index @c PRSTATUS notes.
+	 * See @ref drgn_program_find_prstatus().
+	 */
+	struct drgn_thread_set thread_set;
 	struct drgn_thread *main_thread;
 	struct drgn_thread *crashed_thread;
 	/*
@@ -165,8 +167,12 @@ struct drgn_program {
 		uint64_t mem_section_length;
 		/** VA_BITS on AArch64. */
 		uint64_t va_bits;
+		/** phys_base on x86_64 */
+		uint64_t phys_base;
 		/** Whether 5-level paging was enabled on x86-64. */
 		bool pgtable_l5_enabled;
+		/** Whether CRASHTIME was in the VMCOREINFO. */
+		bool have_crashtime;
 		/** PAGE_SHIFT of the kernel (derived from PAGE_SIZE). */
 		int page_shift;
 
@@ -245,11 +251,6 @@ struct drgn_error *drgn_program_init_kernel(struct drgn_program *prog);
  */
 struct drgn_error *drgn_program_init_pid(struct drgn_program *prog, pid_t pid);
 
-struct drgn_error *
-drgn_program_add_object_finder_impl(struct drgn_program *prog,
-				    struct drgn_object_finder *finder,
-				    drgn_object_find_fn fn, void *arg);
-
 static inline struct drgn_error *
 drgn_program_is_little_endian(struct drgn_program *prog, bool *ret)
 {
@@ -328,28 +329,17 @@ struct drgn_error *drgn_thread_dup_internal(const struct drgn_thread *thread,
 void drgn_thread_deinit(struct drgn_thread *thread);
 
 /**
- * Find the @c NT_PRSTATUS note for the given CPU.
+ * Find the @c NT_PRSTATUS note with the given "PID".
  *
- * This is only valid for the Linux kernel.
+ * For userspace, the PID is the thread ID. For the kernel, it's complicated;
+ * see drgn_get_initial_registers_from_kernel_core_dump().
  *
  * @param[out] ret Returned note data. If not found, <tt>ret->str</tt> is set to
  * @c NULL and <tt>ret->len</tt> is set to zero.
  */
-struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
-						     uint32_t cpu,
-						     struct nstring *ret);
-
-/**
- * Find the @c NT_PRSTATUS note for the given thread ID.
- *
- * This is only valid for userspace programs.
- *
- * @param[out] ret Returned note data. If not found, <tt>ret->str</tt> is set to
- * @c NULL and <tt>ret->len</tt> is set to zero.
- */
-struct drgn_error *drgn_program_find_prstatus_by_tid(struct drgn_program *prog,
-						     uint32_t tid,
-						     struct nstring *ret);
+struct drgn_error *drgn_program_find_prstatus(struct drgn_program *prog,
+					      uint32_t tid,
+					      struct nstring *ret);
 
 /**
  * Cache the @c NT_PRSTATUS note provided by @p data in @p prog.
@@ -364,18 +354,38 @@ struct drgn_error *drgn_program_cache_prstatus_entry(struct drgn_program *prog,
 						     uint32_t *ret);
 
 /*
- * Like @ref drgn_program_find_symbol_by_address(), but @p ret is already
- * allocated, we may already know the module, and doesn't return a @ref
- * drgn_error.
+ * Like @ref drgn_program_find_symbol_by_address(), but returns @c NULL rather
+ * than a lookup error if the symbol was not found.
  *
- * @param[in] module Module containing the address. May be @c NULL, in which
- * case this will look it up.
- * @return Whether the symbol was found.
+ * @param[in] address Address to search for.
+ * @param [out] ret The symbol found by the lookup (if found)
+ * @return @c NULL unless an error (unrelated to a lookup error) was encountered
  */
-bool drgn_program_find_symbol_by_address_internal(struct drgn_program *prog,
-						  uint64_t address,
-						  Dwfl_Module *module,
-						  struct drgn_symbol *ret);
+struct drgn_error *
+drgn_program_find_symbol_by_address_internal(struct drgn_program *prog,
+					     uint64_t address,
+					     struct drgn_symbol **ret);
+
+struct drgn_error *
+drgn_program_register_type_finder_impl(struct drgn_program *prog,
+				       struct drgn_type_finder *finder,
+				       const char *name,
+				       const struct drgn_type_finder_ops *ops,
+				       void *arg, size_t enable_index);
+
+struct drgn_error *
+drgn_program_register_object_finder_impl(struct drgn_program *prog,
+					 struct drgn_object_finder *finder,
+					 const char *name,
+					 const struct drgn_object_finder_ops *ops,
+					 void *arg, size_t enable_index);
+
+struct drgn_error *
+drgn_program_register_symbol_finder_impl(struct drgn_program *prog,
+					 struct drgn_symbol_finder *finder,
+					 const char *name,
+					 const struct drgn_symbol_finder_ops *ops,
+					 void *arg, size_t enable_index);
 
 /**
  * Call before a blocking (I/O or long-running) operation.
