@@ -173,7 +173,7 @@ typedef struct {
 
 typedef struct {
 	PyObject_HEAD
-	Program *prog;
+	PyObject *name_obj; /* object owning the reference to the symbol name */
 	struct drgn_symbol *sym;
 } Symbol;
 
@@ -196,6 +196,16 @@ typedef struct {
 	union drgn_lazy_object *lazy_obj;
 } LazyObject;
 
+typedef struct {
+	PyObject_HEAD
+	uint64_t kinds;
+} TypeKindSet;
+
+typedef struct {
+	PyObject_HEAD
+	uint64_t mask;
+} TypeKindSetIterator;
+
 typedef struct {
 	LazyObject lazy_obj;
 	PyObject *name;
@@ -236,6 +246,8 @@ extern PyTypeObject Symbol_type;
 extern PyTypeObject Thread_type;
 extern PyTypeObject ThreadIterator_type;
 extern PyTypeObject TypeEnumerator_type;
+extern PyTypeObject TypeKindSet_type;
+extern PyTypeObject TypeKindSetIterator_type;
 extern PyTypeObject TypeMember_type;
 extern PyTypeObject TypeParameter_type;
 extern PyTypeObject TypeTemplateParameter_type;
@@ -259,6 +271,9 @@ PyObject *Language_wrap(const struct drgn_language *language);
 int language_converter(PyObject *o, void *p);
 int add_languages(void);
 
+PyObject *TypeKindSet_wrap(uint64_t mask);
+int init_type_kind_set(void);
+
 static inline DrgnObject *DrgnObject_alloc(Program *prog)
 {
 	DrgnObject *ret = call_tp_alloc(DrgnObject);
@@ -288,7 +303,7 @@ Program *program_from_core_dump(PyObject *self, PyObject *args, PyObject *kwds);
 Program *program_from_kernel(PyObject *self);
 Program *program_from_pid(PyObject *self, PyObject *args, PyObject *kwds);
 
-PyObject *Symbol_wrap(struct drgn_symbol *sym, Program *prog);
+PyObject *Symbol_wrap(struct drgn_symbol *sym, PyObject *name_obj);
 
 PyObject *Thread_wrap(struct drgn_thread *drgn_thread);
 
