@@ -68,6 +68,12 @@ py_long_to_bytes_for_object_type(PyObject *value_obj,
 		PyErr_NoMemory();
 		return NULL;
 	}
+#if PY_VERSION_HEX >= 0x030d00a4
+	Py_ssize_t r = PyLong_AsNativeBytes(long_obj, buf, size,
+					    type->little_endian);
+	if (r < 0)
+		return NULL;
+#else
 	// _PyLong_AsByteArray() still returns the least significant bytes on
 	// OverflowError unless the object is negative and is_signed is false.
 	// So, we always pass is_signed as true.
@@ -85,6 +91,7 @@ py_long_to_bytes_for_object_type(PyObject *value_obj,
 			return NULL;
 		}
 	}
+#endif
 	return_ptr(buf);
 }
 
@@ -822,13 +829,12 @@ static PyObject *DrgnObject_repr(DrgnObject *self)
 	if (!parts)
 		return NULL;
 
-	char *type_name;
+	_cleanup_free_ char *type_name = NULL;
 	err = drgn_format_type_name(drgn_object_qualified_type(&self->obj),
 				    &type_name);
 	if (err)
 		return set_drgn_error(err);
 	_cleanup_pydecref_ PyObject *tmp = PyUnicode_FromString(type_name);
-	free(type_name);
 	if (!tmp)
 		return NULL;
 
@@ -880,17 +886,12 @@ static PyObject *DrgnObject_repr(DrgnObject *self)
 static PyObject *DrgnObject_str(DrgnObject *self)
 {
 	struct drgn_error *err;
-	char *str;
-	PyObject *ret;
-
+	_cleanup_free_ char *str = NULL;
 	err = drgn_format_object(&self->obj, SIZE_MAX,
 				 DRGN_FORMAT_OBJECT_PRETTY, &str);
 	if (err)
 		return set_drgn_error(err);
-
-	ret = PyUnicode_FromString(str);
-	free(str);
-	return ret;
+	return PyUnicode_FromString(str);
 }
 
 struct format_object_flag_arg {
@@ -948,8 +949,6 @@ static PyObject *DrgnObject_format(DrgnObject *self, PyObject *args,
 	struct format_object_flag_arg name##_arg = { &flags, value };
 	FLAGS
 #undef X
-	char *str;
-	PyObject *ret;
 
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "|$"
 #define X(name, value) "O&"
@@ -972,13 +971,11 @@ static PyObject *DrgnObject_format(DrgnObject *self, PyObject *args,
 			return NULL;
 	}
 
+	_cleanup_free_ char *str = NULL;
 	err = drgn_format_object(&self->obj, columns, flags, &str);
 	if (err)
 		return set_drgn_error(err);
-
-	ret = PyUnicode_FromString(str);
-	free(str);
-	return ret;
+	return PyUnicode_FromString(str);
 
 #undef FLAGS
 }
