@@ -15,6 +15,7 @@
 #include <assert.h>
 
 #include "drgn.h"
+#include "handler.h"
 #include "hash_table.h"
 #include "vector.h"
 
@@ -50,21 +51,11 @@ drgn_byte_order_from_little_endian(bool little_endian)
 
 /** Registered type finding callback in a @ref drgn_program. */
 struct drgn_type_finder {
-	/** The callback. */
-	drgn_type_find_fn fn;
-	/** Argument to pass to @ref drgn_type_finder::fn. */
+	struct drgn_handler handler;
+	struct drgn_type_finder_ops ops;
 	void *arg;
-	/** Next callback to try. */
-	struct drgn_type_finder *next;
-	/** Whether this structure needs to be freed. */
-	bool free;
 };
 
-struct drgn_error *
-drgn_program_add_type_finder_impl(struct drgn_program *prog,
-				  struct drgn_type_finder *finder,
-				  drgn_type_find_fn fn, void *arg);
-
 DEFINE_HASH_SET_TYPE(drgn_dedupe_type_set, struct drgn_type *);
 
 /** <tt>(type, member name)</tt> pair. */
