@@ -7,6 +7,10 @@ set -eux
 # Drop into a shell if something fails.
 trap 'if [ $? -ne 0 ]; then exec bash -i; fi' EXIT
 
+sed -i -e 's/mirrorlist/#mirrorlist/g' \
+	-e 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' \
+	/etc/yum.repos.d/CentOS-*
+
 yum install -y \
 	bzip2-devel \
 	libzstd-devel \
@@ -53,6 +57,16 @@ curl -L "$libkdumpfile_url" | tar -xz --strip-components=1
 make -j$(($(nproc) + 1))
 make install
 
+# CentOS 7 has check 0.9.9, which is too old.
+check_version=0.15.2
+check_url=https://github.com/libcheck/check/releases/download/$check_version/check-$check_version.tar.gz
+mkdir /tmp/check
+cd /tmp/check
+curl -L "$check_url" | tar -xz --strip-components=1
+./configure --disable-build-docs
+make -j$(($(nproc) + 1))
+make install
+
 ldconfig
 
 mkdir /tmp/drgn
@@ -87,7 +101,8 @@ for pybin in /opt/python/cp*/bin; do
 	if build_for_python "$pybin/python"; then
 		"$pybin/pip" install drgn --no-index -f /tmp/manylinux_wheels/
 		"$pybin/drgn" --version
-		"$pybin/python" setup.py test
+		"$pybin/pip" install pytest
+		"$pybin/pytest"
 	fi
 done
 
