@@ -4,11 +4,11 @@
 import os
 import unittest
 
-from drgn import Object, Program
+from drgn import Object, Program, reinterpret
 from tests import assertReprPrettyEqualsStr, modifyenv
 from tests.linux_kernel import (
     LinuxKernelTestCase,
-    fork_and_sigwait,
+    fork_and_stop,
     skip_unless_have_stack_tracing,
     skip_unless_have_test_kmod,
 )
@@ -16,7 +16,7 @@ from util import NORMALIZED_MACHINE_NAME
 
 
 @skip_unless_have_stack_tracing
-class TestStackTrace(LinuxKernelTestCase):
+class LinuxKernelStackTraceTestCase(LinuxKernelTestCase):
     def _test_drgn_test_kthread_trace(self, trace):
         for i, frame in enumerate(trace):
             if frame.name == "drgn_test_kthread_fn3":
@@ -26,6 +26,8 @@ class TestStackTrace(LinuxKernelTestCase):
         self.assertEqual(trace[i + 1].name, "drgn_test_kthread_fn2")
         self.assertEqual(trace[i + 2].name, "drgn_test_kthread_fn")
 
+
+class TestStackTrace(LinuxKernelStackTraceTestCase):
     @skip_unless_have_test_kmod
     def test_by_task_struct(self):
         self._test_drgn_test_kthread_trace(
@@ -63,6 +65,23 @@ class TestStackTrace(LinuxKernelTestCase):
         self._test_drgn_test_kthread_trace(self.prog.stack_trace(pt_regs))
         self._test_drgn_test_kthread_trace(self.prog.stack_trace(pt_regs.address_of_()))
 
+    @skip_unless_have_test_kmod
+    def test_stack_trace_from_pcs(self):
+        if not self.prog["drgn_test_have_stacktrace"]:
+            self.skipTest("kernel was not built with CONFIG_STACKTRACE")
+        entries = self.prog["drgn_test_stack_entries"]
+        self._test_drgn_test_kthread_trace(
+            self.prog.stack_trace_from_pcs(
+                reinterpret(
+                    self.prog.array_type(
+                        entries.type_.type,
+                        self.prog["drgn_test_num_stack_entries"].value_(),
+                    ),
+                    entries,
+                ).value_()
+            )
+        )
+
     @skip_unless_have_test_kmod
     def test_local_variable(self):
         for frame in self.prog.stack_trace(self.prog["drgn_test_kthread"]):
@@ -88,7 +107,7 @@ class TestStackTrace(LinuxKernelTestCase):
     def test_registers(self):
         # Smoke test that we get at least one register and that
         # StackFrame.registers() agrees with StackFrame.register().
-        with fork_and_sigwait() as pid:
+        with fork_and_stop() as pid:
             trace = self.prog.stack_trace(pid)
             have_registers = False
             for frame in trace:
@@ -100,7 +119,7 @@ class TestStackTrace(LinuxKernelTestCase):
     def test_sp(self):
         # Smoke test that the stack pointer register shows up in
         # StackFrame.registers().
-        with fork_and_sigwait() as pid:
+        with fork_and_stop() as pid:
             trace = self.prog.stack_trace(pid)
             self.assertIn(trace[0].sp, trace[0].registers().values())
 
@@ -111,7 +130,7 @@ class TestStackTrace(LinuxKernelTestCase):
         )
 
     def test_stack__repr_pretty_(self):
-        with fork_and_sigwait() as pid:
+        with fork_and_stop() as pid:
             trace = self.prog.stack_trace(pid)
             assertReprPrettyEqualsStr(trace)
             for frame in trace:
