@@ -106,6 +106,34 @@ _PATCHES = (
         name="sched-work-around-mystery-QEMU-hang.patch",
         versions=((KernelVersion("5.11"), KernelVersion("5.14")),),
     ),
+    _Patch(
+        name="5.4-arm64-build-Remove-.eh_frame-sections-due-to-unwind-.patch",
+        versions=((KernelVersion("5.4"), KernelVersion("5.5")),),
+    ),
+    _Patch(
+        name="4.19-arm64-build-Remove-.eh_frame-sections-due-to-unwind-.patch",
+        versions=((KernelVersion("4.19.15"), KernelVersion("4.20")),),
+    ),
+    _Patch(
+        name="4.14-arm64-build-Remove-.eh_frame-sections-due-to-unwind-.patch",
+        versions=((KernelVersion("4.14.93"), KernelVersion("4.15")),),
+    ),
+    _Patch(
+        name="4.9-arm64-build-Remove-.eh_frame-sections-due-to-unwind-.patch",
+        versions=((KernelVersion("4.9"), KernelVersion("4.10")),),
+    ),
+    _Patch(
+        name="powerpc-pseries-Fix-hcall-tracepoints-with-JUMP_LABE.patch",
+        versions=((KernelVersion("6.2"), KernelVersion("6.4")),),
+    ),
+    _Patch(
+        name="s390-kernel-emit-CFI-data-in-.debug_frame-and-discar.patch",
+        versions=((None, KernelVersion("4.15")),),
+    ),
+    _Patch(
+        name="s390-crash-fix-proc-vmcore-reads.patch",
+        versions=((KernelVersion("5.18"), KernelVersion("6.0")),),
+    ),
 )
 
 
@@ -330,11 +358,21 @@ class KBuild:
             "tools/bpf/resolve_btfids/resolve_btfids",
             "tools/objtool/objtool",
         ]
-        # Before Linux kernel commit bca8f17f57bd ("arm64: Get rid of
-        # asm/opcodes.h") (in v4.10), AArch64 includes this file from 32-bit
-        # Arm.
         if self._arch.name == "aarch64":
-            files.append("arch/arm/include/asm/opcodes.h")
+            # Before Linux kernel commits bca8f17f57bd ("arm64: Get rid of
+            # asm/opcodes.h") and 2fbadc3002c5 ("arm/arm64: xen: Move shared
+            # architecture headers to include/xen/arm" (both in v4.10), AArch64
+            # included these files from 32-bit Arm.
+            files.extend(
+                (
+                    "arch/arm/include/asm/opcodes.h",
+                    "arch/arm/include/asm/xen/hypercall.h",
+                    "arch/arm/include/asm/xen/hypervisor.h",
+                    "arch/arm/include/asm/xen/interface.h",
+                    "arch/arm/include/asm/xen/page-coherent.h",
+                    "arch/arm/include/asm/xen/page.h",
+                )
+            )
         # Before Linux kernel commit efe0160cfd40 ("powerpc/64: Linker
         # on-demand sfpr functions for modules") (in v4.13), this must be
         # available to link into modules.
